
//
		processor 16C505
		include	"macros.ext"
		include	"16c505.ext"

		__config H'fd4'		; disable code-protection, reset, watchdog, use internal RC clock, RB4 is available

ON		equ	1
OFF		equ	0

; Data area
temp0		equ	H'08'		; temp for use by any routine

bitCount	equ	H'09'

charTemp	equ	H'0A'		; character to be sent

delayTemp	equ	H'0B'
delayCnt	equ	H'0C'


;                      |  1st half  |  2nd half  |  3rd half  |
;                      |  bit time  |  bit time  |  bit time  |
;                      |            |            |            |
;              --------|------------|------------|------------|
;               START  |PPPPPP      |PPPPPP      |PPPPPP      |
;                      |            |            |            |
;              --------|------------|------------|-------------
;                ONE   |PPPPPP      |            |
;                      |            |            |
;              --------|------------|------------|
;               ZERO   |            |PPPPPP      |
;                      |            |            |
;              --------|------------|------------|
;
; bit 7 is sent first, followed by bit 6, bit 5, etc.
;

; length of various bits (in pulses at 26 us/pulse)
FULLBITLEN	equ		D'144'			; 4000 us
HALFBITLEN	equ		D'77'			; 2000 us
BURSTLEN	equ		D'38'			; 1000 us

; PORTB bits
YESBIT		equ		0			; RB0 is YES key
NOBIT		equ		1			; RB1 is NO key

; PORTC bits
IROUTBIT	equ		0
STATUSLEDBIT	equ		1

; macros

SENDMARK	macro		numPulses
		movlw		numPulses
		call		SendMark
		endm

SENDSPACE	macro		numPulses
		movlw		numPulses
		call		SendSpace
		endm

SENDSTART	macro
		repeat		3
		SENDMARK	BURSTLEN		; send a burst at the front of three consecutive half-frames
		SENDSPACE	HALFBITLEN-BURSTLEN
		endr
		endm

SENDBIT		macro		bit
		if		bit
		SENDMARK	BURSTLEN		; 1: send a burst at the front of a half-frame
		SENDSPACE	FULLBITLEN-BURSTLEN	;    skip the rest of the half-frame and all of the next
		else
		SENDSPACE	HALFBITLEN		; 0: skip a half frame
		SENDMARK	BURSTLEN		;    send a burst at the start of the next half frame
		SENDSPACE	HALFBITLEN-BURSTLEN	;    skip the rest of the second half frame
		endif
		endm

DELAYMS		macro		time			; wait 'time' ms
		movlw		time
		call		Delay
		endm

SENDCHAR	macro		char			; send 'char' out via IR
		movlw		char
		call		SendChar
		endm
		

		org		0			; starting location
Reset:
		movwf		OSCCAL			; write calibration value

		movlw		B'00000000'		; wake up on pin change, enable weak pullups
		option					; set the option register
		
		movlw		B'00000000'		; make C all outputs
		tris		PORTC
		
		movlw		B'00000011'		; make RB0 and RB1 inputs
		tris		PORTB
Main:
		btfss		PORTB,YESBIT
		goto		SendYes			; yes is pressed
		btfss		PORTB,NOBIT
		goto		SendNo			; no is pressed

		movfw		PORTB			; get current inputs to compare while sleeping
		sleep					; neither pressed, go back to sleep
		
SendYes
;		SENDCHAR	H'59'			; ASCII 'Y' for yes
		SENDMARK	FULLBITLEN		; send a start bit
		goto		SendDone
SendNo
;		SENDCHAR	H'4e'			; ASCII 'N' for no
		SENDMARK	HALFBITLEN		; send a start bit

SendDone
		DELAYMS		D'100'			; wait 100 milliseconds and send again
		goto		Main



;------------------------------------------------------------------------------
;
; send a pulse stream at about 38.5 kHz
; the number of pulses to be sent is passed in W
; (max bit time = 256 pulses * 26 us/pulse = 6.656 ms)
; if W = 0, 256 pulses are sent
;

SendMark:
		movwf		temp0			; use temp0 to count the pulses
		bcf		PORTC,STATUSLEDBIT	; turn on visible status LED

; send a pulse.  for each pulse the LED is on for 13 cycles, then
; off for 13 cycles (1MHz/26 cycles = 38.46 kHz)
;
SendPulse
		bcf		PORTC,IROUTBIT		; (1) drive low (turn on LED)
		repeat		D'12'			; (12)
		nop
		endr					; = 13

		bsf		PORTC,IROUTBIT		; (1) drive high (turn off LED)
		repeat		D'9'
		nop					; (9)
		endr
		decfsz		temp0,F			; (1)
		goto		SendPulse		; (2) = 13

		bsf		PORTC,STATUSLEDBIT	; turn off visible status LED
		retlw		0


;------------------------------------------------------------------------------
;
; send space (no pulses) -- delay some number of pulse times
; the number of pulses to be sent is passed in W
; (max bit time = 256 pulses * 26 us/pulse = 6.656 ms)
; if W = 0, 256 pulses are sent
;

SendSpace:
		movwf		temp0			; use temp0 to count the pulses

; wait a pulse (26 cycles each)
;
WaitPulse
		repeat		D'23'			; (23)
		nop
		endr

		decfsz		temp0,F			; (1)
		goto		WaitPulse		; (2) = 26

		retlw		0


;------------------------------------------------------------------------------
;
; wait one ms
;
Delay1ms
		movlw		D'250'
		movwf		delayTemp
Del1msLoop
		nop					; 1
		decfsz		delayTemp,F		; 1 (usually)
		goto		Del1msLoop		; 2 = 4 us loop x 250 loops = 1000 us = 1 ms
		retlw		0


;------------------------------------------------------------------------------
;
; wait a specified number of ms
; number of ms to delay passed in W
; 0 = 256 ms
;
Delay:
		movwf		delayCnt
DelayLoop
		call		Delay1ms		; wait 1 ms
		decfsz		delayCnt,F		; count number of ms waited, exit if done
		goto		DelayLoop		; wait another
		retlw		0


;------------------------------------------------------------------------------
;
; send a character
; pass the character to send in W
;
SendChar:
		movwf		charTemp		; save the character to be sent
		movlw		D'8'
		movwf		bitCount		; number of bits to send

		SENDSTART				; send a start bit

SendLoop
		btfsc		charTemp,7		; send MSB first
		goto		SendChar1
SendChar0
		SENDBIT		0
		goto		SendNextBit

SendChar1
		SENDBIT		1

SendNextBit
		rlf		charTemp,F		; shift up the next bit to send
		decfsz		bitCount,F		; count sent bits, done if zero
		goto		SendLoop		; send another bit

		retlw		0

		org		0x3ff			; starting location
		movlw		0x005
		end
